unit FavExptImpl;

interface

uses
  FavMenuData, FavEditor,
  menus, toolsapi, actnlist;

type
  TFavouritesMenuExt = class(TObject)
  private
    fFavMenuItem   : TMenuItem;
    fAddFileAction : TAction;
    fAddProjAction : TAction;
    fEditFavAction : TAction;
    Services       : IOTAServices;
    fConstantMenuItems:integer;
    fData           : TFavData;
    fFavEditor      : TFavEditor;
    fLastProjFolder : string;
    fPASImage       : Integer;
    fDPRImage       : Integer;
    fFileImage      : Integer;
  private
    procedure OnFileNameClick(sender:TObject);
    procedure OnAddFile(sender:TObject);
    procedure OnAddProj(sender:TObject);
    procedure OnEditFavs(sender:TObject);
    procedure OnMenuPopup(sender:TObject);
    procedure UpdateActions(sender:TObject);
  protected
    procedure BuildMenu;
    procedure UpdateMenu(force:boolean);
  public
    constructor Create;
    destructor Destroy; override;
    procedure  RegisterCommand;
    procedure  UnregisterCommand;
  end;

implementation

{$R FavExptImpl.res}

uses
  windows, classes, sysutils, graphics;

resourcestring
  sFavMenuCaption = 'Favourites ...';
  sFavAddFileCaption = 'Add &file %s';
  sFavAddProjCaption = 'Add &project %s';
  sFavEditCaption    = '&Edit Favourites ...';
//------------------------------------------------------------------------------

var
  MyIdeExtension: TFavouritesMenuExt = nil;

function GetImageIndex(const resname:string; BackGround:TColor=clPurple):integer;
  var ImageBmp:TBitmap;
begin
  if ResName<>'' then
  begin
    ImageBmp := TBitmap.Create;
    try
      ImageBmp.LoadFromResourceName(FindResourceHInstance(HInstance), ResName);
      result := (BorlandIDEServices as INTAServices).AddMasked(ImageBmp, BackGround);
    finally
      ImageBmp.Free;
    end;
  end

  else
  begin
    result := -1;
  end;
end;

constructor TFavouritesMenuExt.Create;
begin
  inherited Create;
  Services := BorlandIDEServices as IOTAServices;

  fData := TFavData.create;

  RegisterCommand;
  UpdateMenu(true);
end;

destructor TFavouritesMenuExt.Destroy;
begin
  UnregisterCommand;

  fFavEditor.Free;
  fData.Free;

  inherited;
end;

procedure TFavouritesMenuExt.BuildMenu;
var
  IDEActionList : TActionList;
  mi:TMenuItem;
  function NewAction(const name,category,caption:string; ActExecute,ActUpdate:TNotifyEvent; const ImgResName:string=''):TAction ;
  begin
    result := TAction.create(nil);
    result.Name       := name;
    result.category    := category;
    result.Caption    := caption;
    result.Visible    := True;
    result.OnExecute  := ActExecute;
    result.OnUpdate   := ActUpdate;
    result.ImageIndex := GetImageIndex(ImgResName);
    result.ActionList := IDEActionList;
  end;

begin
  IDEActionList := TActionList((BorlandIDEServices as INTAServices).ActionList);

  {Favourites Menu}
  fFavMenuItem := NewItem(sFavMenuCaption,0,false,true,nil,0,'FavouriteFilesItem');
  fFavMenuItem.ImageIndex := GetImageIndex('FAVICON');
  fFavMenuItem.OnClick := OnMenuPopup;

  {Seperator}
  fFavMenuItem.Add(NewLine);

  {Add File}
  fAddFileAction :=   NewAction('FavAddFile', 'File', sFavAddFileCaption,
                                OnAddFile, UpdateActions, 'FAVADDFILE' );
  mi := TMenuItem.create(fFavMenuItem);
  mi.Action := fAddFileAction;
  fFavMenuItem.add(mi);

  {Add Project File}
  fAddProjAction :=   NewAction('FavAddProj', 'File', sFavAddProjCaption,
                                OnAddProj, nil, 'FAVADDPRJ' );
  mi := TMenuItem.create(fFavMenuItem);
  mi.Action := fAddProjAction;
  fFavMenuItem.add(mi);

  {Edit Favourites}
  fEditFavAction :=   NewAction('FavEdit', 'File', sFavEditCaption,
                                OnEditFavs, nil, 'FAVEDIT' );
  mi := TMenuItem.create(fFavMenuItem);
  mi.Action := fEditFavAction;
  fFavMenuItem.add(mi);

  {Fields}
  fConstantMenuItems := fFavMenuItem.count;
  fDPRImage          := GetImageIndex('DPR');
  fPASImage          := GetImageIndex('PAS');
  fFileImage         := -1;
end;

procedure TFavouritesMenuExt.RegisterCommand;
var
  IDEMainMenu: TMainMenu;
  IDEFileItem: TMenuItem;
  I: Integer;
begin
  BuildMenu;

  {Find File Menu}
  IDEMainMenu := (BorlandIDEServices as INTAServices).MainMenu;
  IDEFileItem := nil;
  with IDEMainMenu do
    for I := 0 to Items.Count - 1 do
      if Items[I].Name = 'FileMenu' then
      begin
        IDEFileItem := Items[I];
        Break;
      end;
  Assert(IDEFileItem <> nil);

  {Add Favourites Menu after OpenProject Item}
  with IDEFileItem do
    for I := 0 to Count - 1 do
      if Items[I].Name = 'FileOpenProjectItem' then
      begin
        IDEFileItem.Insert(I+1, fFavMenuItem);
        System.Break;
      end;
  Assert(fFavMenuItem.Parent <> nil);
end;

//------------------------------------------------------------------------------

procedure TFavouritesMenuExt.UnregisterCommand;
begin
  fFavMenuItem.Free;
  fEditFavAction.Free;
  fAddFileAction.Free;
  fAddProjAction.Free;
end;

//------------------------------------------------------------------------------

procedure TFavouritesMenuExt.UpdateMenu(force:boolean);
 var i:integer; mi:TMenuItem; cnt:integer; fn,ext:string;
begin
  fn :=ExtractFilePath(CurrentProjectFileName);
  if (not force) and (fn = fLastProjFolder) then exit;

  fLastProjFolder := fn;


  {Remove items}
  while fFavMenuItem.count>fConstantMenuItems do
    fFavMenuItem.delete(0);

  {Add Items}
  cnt := 1;
  for i := 0 to fData.count-1 do
  begin
    mi := TMenuItem.create(fFavMenuItem);
    mi.tag := i;
    fn := fData.DisplayValues[i];

    if (fn<>'') and (fn[1]<>'-') then
    begin
      ext := UpperCase(ExtractFileExt(fn));
      if ext='.DPR' then
        mi.ImageIndex := fDPRImage
      else if ext='.PAS' then
        mi.ImageIndex := fPASImage
      else
        mi.ImageIndex := fFileImage;

      mi.caption := Format('%2d %s', [cnt, fn] );
      inc(cnt);
    end

    else
    begin
      mi.caption := '-';
    end;

    mi.OnClick := OnFileNameClick;
    fFavMenuItem.Insert(i, mi);
  end;
end;

procedure TFavouritesMenuExt.OnAddFile(sender: TObject);
begin
  if fData.Add( CurrentFilename) then
  begin
    fData.save;
    UpdateMenu(true);
  end;
end;

procedure TFavouritesMenuExt.OnAddProj(sender: TObject);
begin
  if fData.Add( CurrentProjectFilename) then
  begin
    fData.save;
    UpdateMenu(true);
  end;
end;

procedure TFavouritesMenuExt.OnEditFavs(sender: TObject);
begin
  if not assigned(fFavEditor)
    then fFavEditor := TFavEditor.Create(nil);

  if fFavEditor.Edit(fData) then
  begin
    fData.save;
    UpdateMenu(true);
  end;

end;

procedure TFavouritesMenuExt.UpdateActions(sender: TObject);
 var pfn, fn:string;
begin
  pfn := CurrentProjectFileName;
  fAddProjAction.caption := format(sFavAddProjCaption,
                                   [ExtractFileName(pfn) ] );
  fAddProjAction.enabled := (pfn<>'') and (fData.IndexOf(pfn)<0);

  fn := CurrentFileName;
  fAddFileAction.caption := format(sFavAddFileCaption,
                                   [ExtractFileName(fn) ] );
  fAddFileAction.enabled := (fn<>'') and (fn<>pfn) and (fData.IndexOf(fn)<0);
end;

procedure TFavouritesMenuExt.OnFileNameClick(sender: TObject);
 var fn:string;
begin
  fn := fData.values[ (sender as TComponent).tag ] ;
  (Services as IOTAActionServices).OpenFile( fn );
end;


procedure TFavouritesMenuExt.OnMenuPopup(sender: TObject);
begin
  UpdateMenu(false);
end;

initialization
  MyIdeExtension := TFavouritesMenuExt.Create;

finalization
  FreeAndNil(MyIdeExtension);

end.
